@@ -3,10 +3,10 @@
  * The IMP_Folder:: class provides a set of methods for dealing with folders,
  * accounting for subscription, errors, etc.
  *
- * $Horde: imp/lib/Folder.php,v 1.130.10.12 2005/09/14 15:54:14 jan Exp $
+ * $Horde: imp/lib/Folder.php,v 1.130.10.43 2007/07/18 20:09:50 chuck Exp $
  *
- * Copyright 2000-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2000-2005 Jon Parise <jon@csh.rit.edu>
+ * Copyright 2000-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2000-2007 Jon Parise <jon@csh.rit.edu>
  *
  * See the enclosed file COPYING for license information (GPL). If you did not
  * receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -74,39 +74,35 @@ class IMP_Folder {
      *                associative array containing three values: 'val', with
      *                entire folder name after the server specification;
      *                'label', with the full-length folder name meant for
-     *                display ($prefix has been stripped off, etc); and
-     *                'abbrev', containing a shortened (26 characters max)
-     *                label for display in situations where space is short.
+     *                display and 'abbrev', containing a shortened (26
+     *                characters max) label for display in situations where
+     *                space is short.
      */
     function flist($sub = false, $filter = array())
     {
-        global $conf, $imp, $notification;
+        global $conf, $notification;
+
+        $inbox_entry = array('INBOX' => array('val' => 'INBOX', 'label' => _("Inbox"), 'abbrev' => _("Inbox")));
+
+        if ($_SESSION['imp']['base_protocol'] == 'pop3') {
+            return $inbox_entry;
+        }
 
         $list = array();
-
-        $prefix = $imp['folders'];
-        $hierarchies = $imp['hierarchies'];
-        $namespace = $imp['namespace'];
-
-        /* Make sure we're requesting all folders. */
-        $folders = $prefix . '*';
-
-        /* Figure out which list command to use. */
-        $listcmd = ($sub) ? 'imap_lsub' : 'imap_list';
+        $subidx = intval($sub);
 
         /* Compute values that will uniquely identify this list. */
-        $signature = md5(serialize(array($listcmd, $folders)));
-        $full_signature = md5(serialize(array($listcmd, $prefix, $filter, $hierarchies, $namespace)));
+        $full_signature = md5(serialize(array($subidx, $filter)));
 
         /* Either get the list from the cache, or go to the IMAP server to
            obtain it. */
         if ($conf['server']['cache_folders']) {
             require_once 'Horde/SessionObjects.php';
             $sessionOb = &Horde_SessionObjects::singleton();
-            if (!isset($imp['cache']['folder_cache'])) {
-                $imp['cache']['folder_cache'] = array();
+            if (!isset($_SESSION['imp']['cache']['folder_cache'])) {
+                $_SESSION['imp']['cache']['folder_cache'] = array();
             }
-            $folder_cache = &$imp['cache']['folder_cache'];
+            $folder_cache = &$_SESSION['imp']['cache']['folder_cache'];
             if (isset($folder_cache[$full_signature])) {
                 $data = $sessionOb->query($folder_cache[$full_signature]);
                 if ($data) {
@@ -115,116 +111,64 @@ class IMP_Folder {
             }
         }
 
-        if (isset($this->_listCache[$signature])) {
-            $mailboxes = $this->_listCache[$signature];
+        if (isset($this->_listCache[$subidx])) {
+            $maildelim = $this->_listCache[$subidx]['d'];
+            $mailsort = $this->_listCache[$subidx]['s'];
         } else {
-            $server = IMP::serverString();
-            if (!($mailboxes = $listcmd($imp['stream'], $server, $folders))) {
-                $mailboxes = array();
-            }
-
-            if (is_array($hierarchies)) {
-                foreach ($hierarchies as $val) {
-                    $hboxes = $listcmd($imp['stream'], $server, $val . '*');
-                    if (is_array($hboxes)) {
-                        $mailboxes = array_merge($mailboxes, $hboxes);
-                    }
+            $maildelim = $mailsort = array();
+            foreach ($this->_listFolders($sub) as $val) {
+                /* Strip off the prefix only if we are dealing with a personal
+                 * namespace. */
+                $label = $stripped = substr($val, strpos($val, '}') + 1);
+                $ns_info = IMP::getNamespace($label);
+                if ($ns_info['type'] == 'personal') {
+                    $label = substr($label, strlen($ns_info['name']));
                 }
-            }
-
-            /* Strip the server prefix from the list of mailboxes. */
-            if (!empty($mailboxes)) {
-                $new_mboxes = array();
-                $namespace_len = strlen($namespace);
-                $prefix_len = strlen($prefix);
-                $pos = strpos($mailboxes[0], '}') + 1;
-
-                foreach ($mailboxes as $val) {
-                    $label = $stripped = substr($val, $pos);
-                    if (substr($label, 0, $prefix_len) == $prefix) {
-                        $label = substr($label, $prefix_len);
-                    }
-                    if (substr($label, 0, $namespace_len) == $namespace) {
-                        $label = substr($label, $namespace_len);
-                    }
-                    $new_mboxes[$label] = $stripped;
+                if (strcasecmp('INBOX', $label) == 0) {
+                    continue;
                 }
-
-                $mailboxes = $new_mboxes;
+                $maildelim[$stripped] = $ns_info['delimiter'];
+                $mailsort[$stripped] = $label;
             }
-            $this->_listCache[$signature] = $mailboxes;
+            $this->_listCache[$subidx] = array('d' => $maildelim, 's' => $mailsort);
         }
 
-        if ($imp['base_protocol'] != 'pop3' && !empty($mailboxes)) {
-            $parents = $seen = array();
+        if (!empty($mailsort)) {
+            // TODO: Fix use of delimiter here
+            require_once IMP_BASE . '/lib/IMAP/Sort.php';
+            $delimiter = reset($_SESSION['imp']['namespace']);
+            $imap_sort = &new IMP_IMAP_Sort($delimiter['delimiter']);
+            $imap_sort->sortMailboxes($mailsort, true, true);
 
-            require_once 'Horde/IMAP/Sort.php';
-            $imap_mailboxes = &new IMAP_Sort($GLOBALS['imp']['delimiter']);
-            $imap_mailboxes->sortMailboxesByKey($mailboxes);
+            foreach ($mailsort as $mbox_name => $label) {
+                if (in_array($mbox_name, $filter)) {
+                    continue;
+                }
 
-            foreach ($mailboxes as $label => $mailbox) {
-                if (empty($seen[$mailbox])) {
-                    $seen[$mailbox] = true;
+                if (!($decoded_mailbox = String::convertCharset($mbox_name, 'UTF7-IMAP')) &&
+                    empty($this->_errorCache[$mbox_name])) {
+                    $notification->push(sprintf(_("The folder \"%s\" contains illegal characters in its name. It may cause problems. Please see your system administrator."), $label), 'horde.warning');
+                    $this->_errorCache[$mbox_name] = true;
+                }
 
-                    if (!($decoded_mailbox = String::convertCharset($mailbox, 'UTF7-IMAP'))) {
-                        if (empty($this->_errorCache[$mailbox])) {
-                            $notification->push(sprintf(_("The folder \"%s\" contains illegal characters in its name. It may cause problems. Please see your system administrator."), $label), 'horde.warning');
-                            $this->_errorCache[$mailbox] = true;
-                        }
-                    }
-
-                    /* Ignore INBOX, since we always tack it on first. */
-                    if (strcmp($prefix . 'INBOX', $mailbox) != 0) {
-                        $parts = explode($imp['delimiter'], $label);
-                        $subfolder = array_pop($parts);
-                        $parentparts = $parts;
-                        $pstack = array();
-
-                        while ($i = count($parentparts)) {
-                            $parent = implode($imp['delimiter'], $parentparts);
-                            if (!empty($parent) &&
-                                ($parent != 'INBOX') &&
-                                !isset($list[$parent]) &&
-                                !isset($list[$prefix . $namespace . $parent]) &&
-                                !isset($parents[$parent])) {
-                                $parents[$parent] = true;
-                                $plabel = str_repeat(' ', 4 * ($i - 1)) . String::convertCharset($parts[($i - 1)], 'UTF7-IMAP');
-                                if (strlen($plabel) > 26) {
-                                    $pabbrev = String::substr($plabel, 0, 10) . '...' . String::substr($plabel, -13, 13);
-                                } else {
-                                    $pabbrev = $plabel;
-                                }
-                                $pstack[$parent] = array('val' => '', 'label' => $plabel, 'abbrev' => $pabbrev);
-                            }
-                            array_pop($parentparts);
-                        }
-
-                        if (count($pstack)) {
-                            $pstack = array_reverse($pstack);
-                            foreach ($pstack as $pkey => $pval) {
-                                $list[$pkey] = $pval;
-                            }
-                        }
-
-                        $folded = str_repeat(' ', 4 * count($parts)) . String::convertCharset($subfolder, 'UTF7-IMAP');
-
-                        if (strlen($folded) > 26) {
-                            $abbrev = String::substr($folded, 0, 10) . '...' . String::substr($folded, -13, 13);
-                        } else {
-                            $abbrev = $folded;
-                        }
-
-                        if (($imp['dotfiles'] || (strlen($label) > 0 && ($label[0] != '.' && !strstr($label, $imp['delimiter'] . '.'))))
-                            && !in_array($mailbox, $filter)) {
-                            $list[$mailbox] = array('val' => $mailbox, 'label' => $folded, 'abbrev' => $abbrev);
+                $parts = ($maildelim[$mbox_name]) ? explode($maildelim[$mbox_name], $label) : array($label);
+                $partcount = count($parts);
+                for ($i = 1; $i <= $partcount; $i++) {
+                    $item = implode($maildelim[$mbox_name], array_slice($parts, 0, $i));
+                    if (!isset($list[$item])) {
+                        $abbrev = $folded = str_repeat(' ', 4 * ($i - 1)) . String::convertCharset($parts[($i - 1)], 'UTF7-IMAP');
+                        if (strlen($abbrev) > 26) {
+                            $abbrev = String::substr($abbrev, 0, 10) . '...' . String::substr($abbrev, -13, 13);
                         }
+                        $list[$item] = array('val' => ($i == $partcount) ? $mbox_name : '', 'label' => $folded, 'abbrev' => $abbrev);
                     }
                 }
             }
         }
 
+        /* Add the INBOX on top of list if not in the filter list. */
         if (!in_array('INBOX', $filter)) {
-            array_unshift($list, array('val' => 'INBOX', 'label' => _("Inbox"), 'abbrev' => _("Inbox")));
+            $list = $inbox_entry + $list;
         }
 
         /* Save in cache, if needed. */
@@ -263,25 +207,25 @@ class IMP_Folder {
      */
     function delete($folder_array, $subscribe)
     {
-        global $conf, $imp, $notification;
+        global $conf, $notification;
 
         $server = IMP::serverString();
         $return_value = true;
         $deleted = array();
 
         if ($subscribe) {
-            $subscribed_folders = imap_listsubscribed($imp['stream'], $server, '*');
+            $sub_folders = $this->_listFolders(true);
         }
 
         foreach ($folder_array as $folder) {
-            if (!imap_deletemailbox($imp['stream'], $server . $folder)) {
+            if (!imap_deletemailbox($_SESSION['imp']['stream'], $server . $folder)) {
                 $notification->push(sprintf(_("The folder \"%s\" was not deleted. This is what the server said"), IMP::displayFolder($folder)) .
                                     ': ' . imap_last_error(), 'horde.error');
                 $return_value = false;
             } else {
                 if ($subscribe &&
-                    in_array($server . $folder, $subscribed_folders) &&
-                    !imap_unsubscribe($imp['stream'], $server . $folder)) {
+                    in_array($server . $folder, $sub_folders) &&
+                    !imap_unsubscribe($_SESSION['imp']['stream'], $server . $folder)) {
                     $notification->push(sprintf(_("The folder \"%s\" was deleted but you were not unsubscribed from it."), IMP::displayFolder($folder)), 'horde.warning');
                     $return_value = false;
                 } else {
@@ -289,21 +233,25 @@ class IMP_Folder {
                 }
 
                 $deleted[] = $folder;
+                unset($this->_existsResults[$folder]);
             }
         }
 
         if (!empty($deleted)) {
-            /* Update the IMP_Tree cache. */
+            /* Update the IMAP_Tree cache. */
             require_once IMP_BASE . '/lib/IMAP/Tree.php';
-            $imptree = &IMP_Tree::singleton();
-            if ($imptree) {
-                $imptree->delete($deleted);
+            $imaptree = &IMP_Tree::singleton();
+            if ($imaptree) {
+                $imaptree->delete($deleted);
             }
 
             /* Reset the folder cache. */
             if ($conf['server']['cache_folders']) {
-                unset($imp['cache']['folder_cache']);
+                unset($_SESSION['imp']['cache']['folder_cache']);
             }
+
+            /* Recreate Virtual Folders. */
+            $GLOBALS['imp_search']->sessionSetup();
         }
 
         return $return_value;
@@ -321,16 +269,33 @@ class IMP_Folder {
      */
     function create($folder, $subscribe)
     {
-        global $conf, $imp, $notification;
+        global $conf, $notification;
+
+        /* Check permissions. */
+        if (!IMP::hasPermission('create_folders')) {
+            $message = @htmlspecialchars(_("You are not allowed to create folders."), ENT_COMPAT, NLS::getCharset());
+            if (!empty($conf['hooks']['permsdenied'])) {
+                $message = Horde::callHook('_perms_hook_denied', array('imp:create_folders'), 'horde', $message);
+            }
+            $notification->push($message, 'horde.error', array('content.raw'));
+            return false;
+        } elseif (!IMP::hasPermission('max_folders')) {
+            $message = @htmlspecialchars(sprintf(_("You are not allowed to create more than %d folders."), IMP::hasPermission('max_folders', true)), ENT_COMPAT, NLS::getCharset());
+            if (!empty($conf['hooks']['permsdenied'])) {
+                $message = Horde::callHook('_perms_hook_denied', array('imp:max_folders'), 'horde', $message);
+            }
+            $notification->push($message, 'horde.error', array('content.raw'));
+            return false;
+        }
 
         /* Make sure we are not trying to create a duplicate folder */
-        if ($this->exists($imp['stream'], $folder)) {
+        if ($this->exists($folder)) {
             $notification->push(sprintf(_("The folder \"%s\" already exists"), IMP::displayFolder($folder)), 'horde.warning');
             return false;
         }
 
         /* Attempt to create the mailbox */
-        if (!imap_createmailbox($imp['stream'], IMP::serverString($folder))) {
+        if (!imap_createmailbox($_SESSION['imp']['stream'], IMP::serverString($folder))) {
             $notification->push(sprintf(_("The folder \"%s\" was not created. This is what the server said"), IMP::displayFolder($folder)) .
                             ': ' . imap_last_error(), 'horde.error');
             return false;
@@ -338,42 +303,53 @@ class IMP_Folder {
 
         /* Reset the folder cache. */
         if ($conf['server']['cache_folders']) {
-            unset($imp['cache']['folder_cache']);
+            unset($_SESSION['imp']['cache']['folder_cache']);
         }
 
         /* If the user uses SUBSCRIBE, then add to the subscribe list */
-        if ($subscribe &&
-            !imap_subscribe($imp['stream'], IMP::serverString($folder))) {
+        $res = imap_subscribe($_SESSION['imp']['stream'], IMP::serverString($folder));
+        if ($subscribe && !$res) {
             $notification->push(sprintf(_("The folder \"%s\" was created but you were not subscribed to it."), IMP::displayFolder($folder)), 'horde.warning');
         } else {
             /* The folder creation has been successful */
             $notification->push(sprintf(_("The folder \"%s\" was successfully created."), IMP::displayFolder($folder)), 'horde.success');
         }
 
-        /* Update the IMP_Tree object. */
+        /* Update the IMAP_Tree object. */
         require_once IMP_BASE . '/lib/IMAP/Tree.php';
-        $imptree = &IMP_Tree::singleton();
-        if ($imptree) {
-            $imptree->insert($folder);
+        $imaptree = &IMP_Tree::singleton();
+        if ($imaptree) {
+            $imaptree->insert($folder);
         }
 
+        /* Recreate Virtual Folders. */
+        $GLOBALS['imp_search']->sessionSetup();
+
         return true;
     }
 
     /**
      * Finds out if a specific folder exists or not.
      *
-     * @param string $folder    The full utf encoded folder name to be checked.
-     * @param boolean $recheck  Should we recheck the folder?
+     * @param string $folder  The full utf encoded folder name to be checked.
      *
      * @return boolean  Whether or not the folder exists.
      */
-    function exists($folder, $recheck = false)
+    function exists($folder)
     {
-        if ($recheck || !isset($this->_existsResults[$folder])) {
-            $path = IMP::serverString($folder);
-            $res = imap_list($GLOBALS['imp']['stream'], '', $path);
-            $this->_existsResults[$folder] = ($res ? in_array($path, $res) : false);
+        /* Try the IMAP_Tree object first. */
+        require_once IMP_BASE . '/lib/IMAP/Tree.php';
+        $imaptree = &IMP_Tree::singleton();
+        if ($imaptree) {
+            $elt = $imaptree->get($folder);
+            if ($elt && !$imaptree->isContainer($elt)) {
+                return true;
+            }
+        }
+
+        if (!isset($this->_existsResults[$folder])) {
+            $res = @imap_list($_SESSION['imp']['stream'], IMP::serverString(), $folder);
+            $this->_existsResults[$folder] = is_array($res);
         }
 
         return $this->_existsResults[$folder];
@@ -390,24 +366,36 @@ class IMP_Folder {
      */
     function rename($old, $new)
     {
-        global $conf, $imp, $notification;
+        global $conf, $notification;
 
+        /* Don't try to rename from or to an empty string. */
+        if (strlen($old) == 0 || strlen($new) == 0) {
+            return false;
+        }
+
+        $namespace_info = IMP::getNamespace($old);
         $server = IMP::serverString();
         $success = true;
         $deleted = $inserted = array();
 
         /* Get list of any folders that are underneath this one. */
         $all_folders = array($server . $old);
-        $folder_list = imap_list($imp['stream'], $server, $old . $imp['delimiter'] . '*');
+        $folder_list = imap_list($_SESSION['imp']['stream'], $server, $old . $namespace_info['delimiter'] . '*');
         if (is_array($folder_list)) {
             $all_folders = array_merge($folder_list, $all_folders);
+
+            /* Sort the folders in reverse order because some IMAP servers
+             * will automatically rename all folders when the base folder is
+             * renamed which will result in error messages if we rename from
+             * the bottom up. */
+            require_once IMP_BASE . '/lib/IMAP/Sort.php';
+            $ns_new = IMP::getNamespace($new);
+            $imap_sort = &new IMP_IMAP_Sort($ns_new['delimiter']);
+            $imap_sort->sortMailboxes($all_folders);
+            $all_folders = array_reverse($all_folders);
         }
 
-        /* Get subscribed status. */
-        $subscribed_folders = @imap_lsub($imp['stream'], $server, $old . '*');
-        if (!$subscribed_folders) {
-            $subscribed_folders = array();
-        }
+        $sub_folders = $this->_listFolders(true);
 
         foreach ($all_folders as $folder_old) {
             $subscribe = false;
@@ -422,20 +410,27 @@ class IMP_Folder {
             $name_new = substr($folder_new, strpos($folder_new, '}') + 1);
 
             /* Unsubscribe from current folder. */
-            if (in_array($folder_old, $subscribed_folders)) {
+            if (in_array($folder_old, $sub_folders)) {
                 $subscribe = true;
-                imap_unsubscribe($imp['stream'], $folder_old);
+                imap_unsubscribe($_SESSION['imp']['stream'], $folder_old);
             }
 
-            if (imap_renamemailbox($imp['stream'], $folder_old, $folder_new)) {
+            if (imap_renamemailbox($_SESSION['imp']['stream'], $folder_old, $folder_new)) {
                 if ($subscribe) {
-                    imap_subscribe($imp['stream'], $folder_new);
+                    imap_subscribe($_SESSION['imp']['stream'], $folder_new);
                 }
 
                 $deleted[] = $name_old;
                 $inserted[] = $name_new;
 
                 $notification->push(sprintf(_("The folder \"%s\" was successfully renamed to \"%s\"."), IMP::displayFolder($name_old), IMP::displayFolder($name_new)), 'horde.success');
+
+                unset($this->_existsResults[$name_old]);
+
+                // Change current mailbox if current mailbox was renamed.
+                if ($_SESSION['imp']['mailbox'] == $name_old) {
+                    $_SESSION['imp']['mailbox'] = $name_new;
+                }
             } else {
                 $notification->push(sprintf(_("Renaming \"%s\" to \"%s\" failed. This is what the server said"), IMP::displayFolder($name_old), IMP::displayFolder($name_new)) . ': ' . imap_last_error(), 'horde.error');
                 $success = false;
@@ -445,16 +440,18 @@ class IMP_Folder {
         if (!empty($deleted)) {
             /* Update the IMP_Tree cache. */
             require_once IMP_BASE . '/lib/IMAP/Tree.php';
-            $imptree = &IMP_Tree::singleton();
-            if ($imptree) {
-                $imptree->delete($deleted);
-                $imptree->insert($inserted);
+            $imaptree = &IMP_Tree::singleton();
+            if ($imaptree) {
+                $imaptree->rename($deleted, $inserted);
             }
 
             /* Reset the folder cache. */
             if ($conf['server']['cache_folders']) {
-                unset($imp['cache']['folder_cache']);
+                unset($_SESSION['imp']['cache']['folder_cache']);
             }
+
+            /* Recreate Virtual Folders. */
+            $GLOBALS['imp_search']->sessionSetup();
         }
 
         return $success;
@@ -471,19 +468,19 @@ class IMP_Folder {
      */
     function subscribe($folder_array)
     {
-        global $conf, $imp, $notification;
+        global $conf, $notification;
 
         $return_value = true;
         $subscribed = array();
 
         if (!is_array($folder_array)) {
-            $notification->push(_("No folders were specified"), 'horde.message');
+            $notification->push(_("No folders were specified"), 'horde.warning');
             return false;
         }
 
         foreach ($folder_array as $folder) {
             if ($folder != ' ') {
-                if (!imap_subscribe($imp['stream'], IMP::serverString($folder))) {
+                if (!imap_subscribe($_SESSION['imp']['stream'], IMP::serverString($folder))) {
                     $notification->push(sprintf(_("You were not subscribed to \"%s\". Here is what the server said"), IMP::displayFolder($folder)) . ': ' . imap_last_error(), 'horde.error');
                     $return_value = false;
                 } else {
@@ -494,16 +491,16 @@ class IMP_Folder {
         }
 
         if (!empty($subscribed)) {
-            /* Initialize the IMP_Tree object. */
+            /* Initialize the IMAP_Tree object. */
             require_once IMP_BASE . '/lib/IMAP/Tree.php';
-            $imptree = &IMP_Tree::singleton();
-            if ($imptree) {
-                $imptree->subscribe($subscribed);
+            $imaptree = &IMP_Tree::singleton();
+            if ($imaptree) {
+                $imaptree->subscribe($subscribed);
             }
 
             /* Reset the folder cache. */
             if ($conf['server']['cache_folders']) {
-                unset($imp['cache']['folder_cache']);
+                unset($_SESSION['imp']['cache']['folder_cache']);
             }
         }
 
@@ -521,7 +518,7 @@ class IMP_Folder {
      */
     function unsubscribe($folder_array)
     {
-        global $conf, $imp, $notification;
+        global $conf, $notification;
 
         $return_value = true;
         $unsubscribed = array();
@@ -534,8 +531,8 @@ class IMP_Folder {
         foreach ($folder_array as $folder) {
             if ($folder != ' ') {
                 if (strcasecmp($folder, 'INBOX') == 0) {
-                    $notification->push(sprintf(_("You can not unsubscribe from \"%s\"."), IMP::displayFolder($folder)), 'horde.error');
-                } elseif (!imap_unsubscribe($imp['stream'], IMP::serverString($folder))) {
+                    $notification->push(sprintf(_("You cannot unsubscribe from \"%s\"."), IMP::displayFolder($folder)), 'horde.error');
+                } elseif (!imap_unsubscribe($_SESSION['imp']['stream'], IMP::serverString($folder))) {
                     $notification->push(sprintf(_("You were not unsubscribed from \"%s\". Here is what the server said"), IMP::displayFolder($folder)) . ': ' . imap_last_error(), 'horde.error');
                     $return_value = false;
                 } else {
@@ -546,16 +543,16 @@ class IMP_Folder {
         }
 
         if (!empty($unsubscribed)) {
-            /* Initialize the IMP_Tree object. */
+            /* Initialize the IMAP_Tree object. */
             require_once IMP_BASE . '/lib/IMAP/Tree.php';
-            $imptree = &IMP_Tree::singleton();
-            if ($imptree) {
-                $imptree->unsubscribe($unsubscribed);
+            $imaptree = &IMP_Tree::singleton();
+            if ($imaptree) {
+                $imaptree->unsubscribe($unsubscribed);
             }
 
             /* Reset the folder cache. */
             if ($conf['server']['cache_folders']) {
-                unset($imp['cache']['folder_cache']);
+                unset($_SESSION['imp']['cache']['folder_cache']);
             }
         }
 
@@ -585,9 +582,9 @@ class IMP_Folder {
             $imp_imap = &IMP_IMAP::singleton();
             foreach ($folder_list as $folder) {
                 $imp_imap->changeMbox($folder, OP_READONLY);
-                $count = imap_num_msg($GLOBALS['imp']['stream']);
+                $count = imap_num_msg($_SESSION['imp']['stream']);
                 for ($i = 1; $i <= $count; $i++) {
-                    $h = imap_header($GLOBALS['imp']['stream'], $i);
+                    $h = imap_header($_SESSION['imp']['stream'], $i);
                     $from = '<>';
                     if (isset($h->from[0])) {
                         if (isset($h->from[0]->mailbox) && isset($h->from[0]->host)) {
@@ -599,13 +596,12 @@ class IMP_Folder {
                        require a space in front of single digit days. */
                     $date = sprintf('%s %2s %s', date('D M', $h->udate), date('j', $h->udate), date('H:i:s Y', $h->udate));
                     $body .= 'From ' . $from . ' ' . $date . "\n";
-                    $body .= imap_fetchheader($GLOBALS['imp']['stream'], $i, FT_PREFETCHTEXT);
-                    $body .= imap_body($GLOBALS['imp']['stream'], $i) . "\n";
+                    $body .= str_replace("\r\n", "\n", imap_fetchheader($_SESSION['imp']['stream'], $i, FT_PREFETCHTEXT));
+                    $body .= str_replace("\r\n", "\n", imap_body($_SESSION['imp']['stream'], $i, FT_PEEK) . "\n");
                 }
             }
         }
 
-        $body = str_replace("\r\n", "\n", $body);
         return $body;
     }
 
@@ -629,13 +625,13 @@ class IMP_Folder {
         while (!feof($fd)) {
             $line = fgets($fd, 4096);
 
-            if (preg_match('/From .+@.+/A', $line)) {
+            if (preg_match('/From (.+@.+|- )/A', $line)) {
                 if (!empty($message)) {
                     // Make absolutely sure there are no bare newlines.
                     $message = preg_replace("|([^\r])\n|", "\\1\r\n", $message);
                     $message = str_replace("\n\n", "\n\r\n", $message);
 
-                    if (imap_append($GLOBALS['imp']['stream'], $target, $message)) {
+                    if (imap_append($_SESSION['imp']['stream'], $target, $message)) {
                         $msgcount++;
                     }
                 }
@@ -651,7 +647,7 @@ class IMP_Folder {
             $message = preg_replace("|([^\r])\n|", "\\1\r\n", $message);
             $message = str_replace("\n\n", "\n\r\n", $message);
 
-            if (imap_append($GLOBALS['imp']['stream'], $target, $message)) {
+            if (imap_append($_SESSION['imp']['stream'], $target, $message)) {
                 $msgcount++;
             }
         }
@@ -659,4 +655,34 @@ class IMP_Folder {
         return ($msgcount > 0) ? $msgcount : false;
     }
 
+    /**
+     * Get a list of all folders on the server (including any folders in
+     * hidden namespaces).
+     *
+     * @access private
+     *
+     * @param boolean $sub  List only subscribed folders?
+     *
+     * @return array  An array of folders names.
+     */
+    function _listFolders($sub = false)
+    {
+        $list = array();
+        $listcmd = ($sub) ? 'imap_lsub' : 'imap_list';
+        $server = IMP::serverString();
+
+        /* According to RFC 3501 [6.3.8], the '*' wildcard doesn't
+         * necessarily match all visible mailboxes.  So we have to go
+         * through each namespace separately, even though we may duplicate
+         * mailboxes. */
+        foreach ($_SESSION['imp']['namespace'] as $val) {
+            $hiddenboxes = $listcmd($_SESSION['imp']['stream'], $server, $val['name'] . '*');
+            if (is_array($hiddenboxes)) {
+                $list = array_unique(array_merge($list, $hiddenboxes));
+            }
+        }
+
+        return array_values($list);
+    }
+
 }
