@@ -118,7 +118,7 @@ define('IMP_IDX_SEP', "\1");
 /**
  * IMP Base Class.
  *
- * $Horde: imp/lib/IMP.php,v 1.449.4.69 2007/02/10 22:47:54 slusarz Exp $
+ * $Horde: imp/lib/IMP.php,v 1.449.4.71 2007/05/04 16:36:38 jan Exp $
  *
  * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
  * Copyright 1999-2007 Jon Parise <jon@horde.org>
@@ -758,7 +758,7 @@ class IMP {
         if ($prefs->getValue('compose_popup') &&
             $browser->hasFeature('javascript')) {
             Horde::addScriptFile('popup.js');
-            return "javascript:popup_imp('" . Horde::applicationUrl('compose.php') . "',700,650,'" . htmlspecialchars($browser->escapeJSCode(addslashes($url))) . "');";
+            return "javascript:popup_imp('" . Horde::applicationUrl('compose.php') . "',800,650,'" . htmlspecialchars($browser->escapeJSCode(addslashes($url))) . "');";
         } else {
             return Horde::applicationUrl(empty($url) ? 'compose.php' : 'compose.php?' . $url);
         }
@@ -1202,8 +1202,7 @@ class IMP {
         }
 
         foreach ($_SESSION['imp']['namespace'] as $key => $val) {
-            $mbx = $mailbox . $val['delimiter'];
-            if (!empty($key) && (strpos($mbx, $key) === 0)) {
+            if (!empty($key) && (strpos($mailbox, $key) === 0)) {
                 $cache[$key][$mailbox] = $val;
                 return $val;
             }
