@@ -15,7 +15,7 @@ define('IMP_MESSAGE_COPY', 2);
  * For any function below that requires an $indices parameter, see
  * IMP::parseIndicesList() for the list of allowable inputs.
  *
- * $Horde: imp/lib/Message.php,v 1.164.8.47 2007/01/02 13:54:56 jan Exp $
+ * $Horde: imp/lib/Message.php,v 1.164.8.49 2007/07/03 14:56:26 slusarz Exp $
  *
  * Copyright 2000-2007 Chris Hyde <chris@jeks.net>
  * Copyright 2000-2007 Chuck Hagenbuch <chuck@horde.org>
@@ -204,17 +204,18 @@ class IMP_Message {
                     $error_msg = imap_last_error();
                     $error = true;
 
-                    /* Handle the case when the mailbox is overquota
-                       (moving message to trash would fail) by first
-                       deleting then copying message to Trash. */
+                    /* Handle the case when the mailbox is overquota (moving
+                     * message to trash would fail) by first deleting then
+                     * copying message to Trash. */
                     if ((stristr($error_msg, 'over quota') !== false) ||
+                        (stristr($error_msg, 'quota exceeded') !== false) ||
                         (stristr($error_msg, 'exceeded your mail quota') !== false)) {
                         $error = false;
                         $msg_text = array();
 
                         /* Get text of deleted messages. */
                         foreach ($msgIndices as $val) {
-                            $msg_text[] = imap_fetchheader($imp['stream'], $val, FT_UID | FT_PREFETCHTEXT);
+                            $msg_text[] = imap_fetchheader($imp['stream'], $val, FT_UID | FT_PREFETCHTEXT) . imap_body($imp['stream'], $val, FT_UID);
                         }
                         @imap_delete($imp['stream'], $sequence, FT_UID);
                         @imap_expunge($imp['stream']);
