@@ -7,9 +7,9 @@ require_once IMP_BASE . '/lib/Compose.php';
  * The IMP_Spam:: class contains functions related to reporting spam
  * messages in IMP.
  *
- * $Horde: imp/lib/Spam.php,v 1.3.4.7 2005/09/04 00:06:07 selsky Exp $
+ * $Horde: imp/lib/Spam.php,v 1.3.4.14 2007/01/02 13:54:56 jan Exp $
  *
- * Copyright 2004-2005 Michael Slusarz <slusarz@curecanti.org>
+ * Copyright 2004-2007 Michael Slusarz <slusarz@curecanti.org>
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -47,7 +47,7 @@ class IMP_Spam {
      * Report a list of messages as spam, based on the local configuration
      * parameters.
      *
-     * @param array &$indices  See IMP::parseIndicesList().
+     * @param mixed &$indices  See IMP::parseIndicesList().
      * @param string $action   Either 'spam' or 'notspam'.
      */
     function reportSpam(&$indices, $action)
@@ -76,9 +76,9 @@ class IMP_Spam {
             $imp_imap->changeMbox($folder);
 
             foreach ($msgIndices as $msgnum) {
-                /* Fetch the raw message contents (headers and complete 
+                /* Fetch the raw message contents (headers and complete
                  * body). */
-                $imp_contents = &IMP_Contents::singleton($msgnum);
+                $imp_contents = &IMP_Contents::singleton($msgnum . IMP_IDX_SEP . $folder);
 
                 $to = null;
                 $report_flag = false;
@@ -90,9 +90,27 @@ class IMP_Spam {
                     $raw_msg = $imp_contents->fullMessageText();
                     /* Use a pipe to write the message contents. This should
                      * be secure. */
-                    $pipe = popen($GLOBALS['conf'][$action]['program'], 'w');
-                    fwrite($pipe, $raw_msg);
-                    pclose($pipe);
+                    $prog = str_replace('%u', escapeshellarg(Auth::getAuth()), $GLOBALS['conf'][$action]['program']);
+                    $proc = proc_open($prog,
+                                      array(0 => array('pipe', 'r'),
+                                            1 => array('pipe', 'w'),
+                                            2 => array('pipe', 'w')),
+                                      $pipes);
+                    if (!is_resource($proc)) {
+                        Horde::logMessage('Cannot open process ' . $prog, __FILE__, __LINE__, PEAR_LOG_ERR);
+                        return;
+                    }
+                    fwrite($pipes[0], $raw_msg);
+                    fclose($pipes[0]);
+                    $stderr = '';
+                    while (!feof($pipes[2])) {
+                        $stderr .= fgets($pipes[2]);
+                    }
+                    fclose($pipes[2]);
+                    if (!empty($stderr)) {
+                        Horde::logMessage('Error reporting spam: ' . $stderr, __FILE__, __LINE__, PEAR_LOG_ERR);
+                    }
+                    proc_close($proc);
                     $report_msg_count++;
                     $report_flag = true;
                 }
@@ -105,7 +123,7 @@ class IMP_Spam {
                     }
                     $this->_sendSpamReportMessage($action, $raw_msg);
                     $email_msg_count++;
-                }
+               }
 
                 /* If a (not)spam bounce email address has been provided, use
                  * it. */
