@@ -1,10 +1,10 @@
 <?php
 /**
- * $Horde: imp/test.php,v 1.33.6.4 2005/04/11 14:37:22 chuck Exp $
+ * $Horde: imp/test.php,v 1.33.6.13 2007/01/02 13:54:54 jan Exp $
  *
- * Copyright 2002-2005 Brent J. Nordquist <bjn@horde.org>
- * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 2002-2007 Brent J. Nordquist <bjn@horde.org>
+ * Copyright 1999-2007 Charles J. Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -104,6 +104,10 @@ $pear_list = array(
     'HTTP_Request' => array(
         'path' => 'HTTP/Request.php',
         'error' => 'The HTML composition mode requires HTTP_Request.'
+    ),
+    'Auth_SASL' => array(
+        'path' => 'Auth/SASL.php',
+        'error' => 'If your IMAP server uses CRAM-MD5 or DIGEST-MD5 authentication, this module is required.'
     )
 );
 
@@ -135,7 +139,7 @@ $pear_output = $horde_test->PEARModuleCheck($pear_list);
     <?php echo $pear_output ?>
 </ul>
 
-<h1>PHP IMAP Support Test</h1>
+<h1>PHP Mail Server Support Test</h1>
 <?php
 
 $server = isset($_POST['server']) ? $_POST['server'] : '';
@@ -147,7 +151,6 @@ $type = isset($_POST['server_type']) ? $_POST['server_type'] : '';
 if (!empty($server) && !empty($user) && !empty($passwd) && !empty($type)) {
     if ($type == 'pop') {
         $conn = array(
-            'pop3' => 110,
             'pop3/notls' => 110,
             'pop3/ssl' => 995,
             'pop3/ssl/novalidate-cert' => 995,
@@ -155,7 +158,6 @@ if (!empty($server) && !empty($user) && !empty($passwd) && !empty($type)) {
         );
     } else {
         $conn = array(
-            'imap' => 143,
             'imap/notls' => 143,
             'imap/ssl' => 993,
             'imap/ssl/novalidate-cert' => 993,
@@ -178,11 +180,19 @@ if (!empty($server) && !empty($user) && !empty($passwd) && !empty($type)) {
                 echo '<font color="green">SUCCESS</font> - INBOX has ', $minfo->Nmsgs, ' messages (' . $minfo->Unread, ' new ', $minfo->Recent, ' recent)';
                 $success[] = array('server' => $server, 'protocol' => $key, 'port' => $server_port);
             } else {
-                echo '<font color="red">ERROR</font> - The server returned the following error message:' . "\n" . '<pre>'; print_r(imap_errors()); echo '</pre>';
+                echo '<font color="red">ERROR</font> - The server returned the following error message:' . "\n" . '<pre>';
+                foreach (imap_errors() as $error) {
+                    echo wordwrap($error);
+                }
+                echo '</pre>';
             }
             @imap_close($mbox);
         } else {
-            echo '<font color="red">ERROR</font> - The server returned the following error message:' . "\n" . '<pre>'; print_r(imap_errors()); echo '</pre>';
+            echo '<font color="red">ERROR</font> - The server returned the following error message:' . "\n" . '<pre>';
+            foreach (imap_errors() as $error) {
+                echo wordwrap($error);
+            }
+            echo '</pre>';
         }
         echo "</blockquote>\n</li></ul>\n";
     }
@@ -191,14 +201,54 @@ if (!empty($server) && !empty($user) && !empty($passwd) && !empty($type)) {
         echo "<strong>The following configurations were successful and may be used in your imp/config/servers.php file:</strong>\n";
         $i = 1;
         foreach ($success as $val) {
-            echo "<blockquote><em>Configuration " . $i++ . "</em><pre>";
+            echo "<blockquote><em>Configuration " . $i++ . "</em><blockquote><pre>";
             foreach ($val as $key => $entry) {
                 echo "'" . $key . "' => '" . $entry . "'\n";
             }
-            echo "</pre></blockquote>\n";
+            echo "</pre></blockquote></blockquote>\n";
+        }
+
+        if ($type == 'imap') {
+            echo "<strong>The following IMAP server information was discovered from the remote server:</strong>\n";
+            $config = reset($success);
+            require_once './lib/IMAP/Client.php';
+            $imapclient = &new IMP_IMAPClient($config['server'], $config['port'], $config['protocol']);
+            $use_tls = $imapclient->useTLS();
+            if (!is_a($use_tls, 'PEAR_Error')) {
+                $res = $imapclient->login($user, $passwd);
+            }
+            if (isset($res) && !is_a($res, 'PEAR_Error')) {
+                echo "<blockquote><em>Namespace Information</em><blockquote><pre>";
+                $namespace = $imapclient->namespace();
+                if (is_array($namespace)) {
+                    foreach ($namespace as $val) {
+                        echo "NAMESPACE: \"" . $val['name'] . "\"\n";
+                        echo "DELIMITER: " . $val['delimiter'] . "\n";
+                        echo "TYPE: " . $val['type'] . "\n\n";
+                    }
+                } else {
+                    echo "Could not retrieve namespace information from IMAP server.\n";
+                }
+                echo "</pre></blockquote></blockquote>\n";
+
+                echo "<blockquote><em>IMAP CHILDREN support:</em><blockquote><pre>";
+                echo ($imapclient->queryCapability('CHILDREN')) ? 'SUPPORTED' : 'Not supported';
+                echo "</pre></blockquote></blockquote>\n";
+
+                echo "<blockquote><em>IMAP Charset Search Support:</em><blockquote><pre>";
+                $charset = NLS::getCharset();
+                if ($imapclient->searchCharset($charset)) {
+                    echo "Server supports searching with the $charset character set.\n";
+                } else {
+                    echo "Server does not support searching with the $charset character set.\n";
+                }
+                echo "</pre></blockquote></blockquote>\n";
+            } else {
+                echo "<blockquote><em>Could not retrieve IMAP information from the remote server.</em></blockquote>";
+            }
         }
     } else {
-        echo "<strong>Could not determine a successful connection protocol.  Make sure your IMAP server is running and you have specified the correct port.</strong>\n";
+        echo "<strong>Could not determine a successful connection protocol.  Make sure your mail server is running and you have specified the correct port.</strong>\n";
     }
 } else {
     ?>
@@ -214,8 +264,5 @@ if (!empty($server) && !empty($user) && !empty($passwd) && !empty($type)) {
 </form>
 <?php } ?>
 
-</td></tr>
-</table>
-
 <?php
 require TEST_TEMPLATES . 'footer.inc';
