@@ -17,11 +17,11 @@
  *                 objects
  *   'zip'      -- Download in .zip format?
  *
- * $Horde: imp/view.php,v 2.199.4.4 2005/09/24 09:09:11 jan Exp $
+ * $Horde: imp/view.php,v 2.199.4.11 2007/01/02 13:54:54 jan Exp $
  *
- * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 1999-2007 Charles J. Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Jon Parise <jon@horde.org>
+ * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (GPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -30,7 +30,7 @@
 /* We can't register this session as 'readonly' if we are doing a
  * 'view_attach' since some MIME_Drivers may need to create temporary cache
  * files to correctly generate links. */
-if (isset($_GET['actionID']) && ($_GET['actionID'] != 'view_attach')) {
+if (isset($_GET['actionID']) && $_GET['actionID'] != 'view_attach') {
     $session_control = 'readonly';
 }
 
@@ -75,12 +75,10 @@ if ($actionID == 'compose_attach_preview') {
         if (empty($index)) {
             exit;
         }
-        $contents = &IMP_Contents::singleton($index);
+        $contents = &IMP_Contents::singleton($index . IMP_IDX_SEP . $_SESSION['imp']['thismailbox']);
     }
 
-    if (($actionID != 'download_all') &&
-        ($actionID != 'save_message') &&
-        ($actionID != 'view_source')) {
+    if (!in_array($actionID, array('download_attach', 'download_all', 'save_message', 'view_source'))) {
         $mime = $contents->getDecodedMIMEPart($id);
         if (($ctype = Util::getFormData('ctype'))) {
             $mime->setType($ctype);
@@ -94,7 +92,7 @@ case 'download_all':
     $tosave = array();
     $headers = &$contents->getHeaderOb();
     $headers->buildHeaders();
-    $zipfile = trim($headers->getValue('subject'));
+    $zipfile = trim(preg_replace('/[^\w-+_\. ]/', '_', $headers->getValue('subject')), ' _');
     if (empty($zipfile)) {
         $zipfile = _("attachments.zip");
     } else {
@@ -104,6 +102,9 @@ case 'download_all':
         $mime = $contents->getDecodedMIMEPart($val);
         $tosave[] = array('data' => $mime->getContents(), 'name' => $mime->getName(true, true));
     }
+    if (is_a($contents, 'PEAR_Error')) {
+        Horde::fatal($contents, __FILE__, __LINE__);
+    }
 
     require_once 'Horde/Compress.php';
     $horde_compress = &Horde_Compress::singleton('zip');
